-------------------------------------------------------------------------------
-- File name : main_control.vhd
-- Title     : Top-level control/microprogram
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity main_control is
  port (op_func     : in  std_logic_vector(6 downto 0);
        wb_ack      : in  std_logic;
        overf       : in  std_logic;
        clk         : in  std_logic;
        reset       : in  std_logic;
        stb_o       : out std_logic;
        causewrite  : out std_logic;
        intcause    : out std_logic;
        epcwrite    : out std_logic;
        regwrite    : out std_logic;
        alusrca     : out std_logic;
        we          : out std_logic;
        iord	    : out std_logic;
        irwrite     : out std_logic;
        pcwrite     : out std_logic;
        pcwritecond : out std_logic;
        pcwbnq	    : out std_logic;
        regdst      : out std_logic_vector(1 downto 0);
        alusrcb     : out std_logic_vector(1 downto 0);
        memtoreg    : out std_logic_vector(1 downto 0);
        aluop       : out std_logic_vector(1 downto 0);
        pcsource    : out std_logic_vector(1 downto 0));
end main_control;

architecture main_control of main_control is

  component control_logic 
    port (irwrite      : in  std_logic;
          wb_ack       : in  std_logic;
          stb_o        : in  std_logic;
          overflow     : in  std_logic;
          mux_ctrl_sel : out std_logic_vector(1 downto 0));
  end component;

  component inst_address 
    port (inst    : in  std_logic_vector(6 downto 0);	
          romaddr : out std_logic_vector(5 downto 0));
  end component;

  component mux21_6 
    port (a   : in  std_logic_vector(5 downto 0);
          b   : in  std_logic_vector(5 downto 0);
          sel : in  std_logic;
          z   : out std_logic_vector(5 downto 0));
  end component;

  component mux41_6 
    port (a   : in  std_logic_vector(5 downto 0);
          b   : in  std_logic_vector(5 downto 0);
          c   : in  std_logic_vector(5 downto 0);
          d   : in  std_logic_vector(5 downto 0);       
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic_vector(5 downto 0));  
  end component;

  component counter6 
    port (addrin	: in  std_logic_vector(5 downto 0);
          addrout	: out std_logic_vector(5 downto 0));
  end component;

  component register6 
    port (d   : in  std_logic_vector (5 downto 0);
          clk : in  std_logic;
          q   : out std_logic_vector (5 downto 0));
  end component;

  component romctrl 
    port (address : in  std_logic_vector(5 downto 0);
          q       : out std_logic_vector(21 downto 0));    
  end component;

  signal irwrite_int, stb_o_int : std_logic;
  signal ctrl_sel : std_logic_vector (1 downto 0);
  signal resetmux_out, zr6, inst_out, counter_out, of_addr, muxaddr_out, romaddr_in: std_logic_vector (5 downto 0);
  signal rom_out : std_logic_vector (21 downto 0);

begin

  zr6 (5 downto 0) <= "000000";
  of_addr(5 downto 0) <= "100100";	-- overflow exception address, 24
  stb_o_int <= rom_out(21);
  irwrite_int <= rom_out(7);
          
  insaddr: inst_address port map ( inst => op_func, romaddr => inst_out);
  resetmux: mux21_6 port map (zr6, muxaddr_out, reset, resetmux_out);
  muxaddr: mux41_6 port map (a => counter_out, b => inst_out, c => romaddr_in, d => of_addr,
                             sel => ctrl_sel, z => muxaddr_out);
  addr_reg: register6 port map (d => resetmux_out, clk => clk, q => romaddr_in);
  countplus1: counter6 port map (addrin => romaddr_in, addrout => counter_out);
  ctrl_log: control_logic port map (irwrite => irwrite_int, wb_ack => wb_ack, stb_o => stb_o_int,
                                    overflow => overf, mux_ctrl_sel => ctrl_sel);
  ctrl_rom: romctrl port map (address => romaddr_in, q => rom_out);

  stb_o <= stb_o_int;
  causewrite <= rom_out(20);
  intcause <= rom_out(19);
  epcwrite <= rom_out(18);
  regdst(1 downto 0) <= rom_out(17 downto 16);
  regwrite <= rom_out(15);
  alusrca <= rom_out(14);
  alusrcb <= rom_out(13 downto 12);
  we <= rom_out(11);
  memtoreg <= rom_out(10 downto 9);
  iord <= rom_out(8);
  irwrite <= irwrite_int;
  pcwrite <= rom_out(6);
  pcwritecond <= rom_out(5);
  pcwbnq <= rom_out(4);
  aluop <= rom_out(3 downto 2);
  pcsource <= rom_out(1 downto 0);	

end main_control;


