-------------------------------------------------------------------------------
-- File name : mips.vhd
-- Title     : The whole kit and kaboodle
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity mipsmc is
  port (clk        :in  std_logic;
        reset_l    :in  std_logic;
        ack_o      :in  std_logic;
        adr_i      :out std_logic_vector(  8 downto 0 );
        clk_i      :out std_logic;
        dat_i      :out std_logic_vector( 31 downto 0 );
        dat_o      :in  std_logic_vector( 31 downto 0 );
        stb_i      :out std_logic;
        we_i       :out std_logic);
end mipsmc;

architecture struct of mipsmc is

  component datapath 
    port (pcwritecond  : in  std_logic;
          pcwbnq       : in  std_logic;
          pcwrite      : in  std_logic;
          iord         : in  std_logic;
          irwrite      : in  std_logic;
          alusrca      : in  std_logic;
          regwrite     : in  std_logic;
          clk          : in  std_logic;
          causewrite   : in  std_logic;
          intcause     : in  std_logic;
          epcwrite     : in  std_logic;
          aluop        : in  std_logic_vector(1 downto 0);
          alusrcb      : in  std_logic_vector(1 downto 0);
          pcsource     : in  std_logic_vector(1 downto 0);
          regdst       : in  std_logic_vector(1 downto 0);
          memtoreg     : in  std_logic_vector(1 downto 0);
          memdata      : in  std_logic_vector(31 downto 0);
          reset        : in  std_logic;
          cause        : out std_logic;
          overflow     : out std_logic;
          op           : out std_logic_vector(6 downto 0);
          address      : out std_logic_vector(8 downto 0);
          memwritedata : out std_logic_vector(31 downto 0);
          epcout       : out std_logic_vector(31 downto 0));
  end component;

  component main_control 
    port (op_func     : in  std_logic_vector(6 downto 0);
          wb_ack      : in  std_logic;
          overf       : in  std_logic;
          clk         : in  std_logic;
          reset       : in  std_logic;
          stb_o       : out std_logic;
          causewrite  : out std_logic;
          intcause    : out std_logic;
          epcwrite    : out std_logic;
          regwrite    : out std_logic;
          alusrca     : out std_logic;
          we          : out std_logic;
          iord	      : out std_logic;
          irwrite     : out std_logic;
          pcwrite     : out std_logic;
          pcwritecond : out std_logic;
          pcwbnq      : out std_logic;
          regdst      : out std_logic_vector(1 downto 0);
          alusrcb     : out std_logic_vector(1 downto 0);
          memtoreg    : out std_logic_vector(1 downto 0);
          aluop       : out std_logic_vector(1 downto 0);
          pcsource    : out std_logic_vector(1 downto 0));
  end component;

  component mux21 
    port (a   : in  std_logic;
          b   : in  std_logic;
          sel : in  std_logic;
          z   : out std_logic);
  end component;

  signal pcwritecond, pcwbnq, pcwrite, pcwrite_reset, iord, irwrite :std_logic;
  signal alusrca, regwrite, causewrite, intcause, epcwrite :std_logic;
  signal aluop, alusrcb, pcsource, regdst, memtoreg :std_logic_vector(1 downto 0);
  signal memdata :std_logic_vector(31 downto 0);
  signal overflow :std_logic;
  signal op :std_logic_vector(6 downto 0);
  signal address :std_logic_vector(8 downto 0);
  signal memwritedata :std_logic_vector(31 downto 0);
  signal  ack_int, stb, we, clk_bar :std_logic;

begin

  clk_bar <= not clk;

  ack_int <= ack_o;
  adr_i <= address;
  clk_i <= clk_bar;
  dat_i <= memwritedata;
  memdata <= dat_o;
  stb_i <= stb;
  we_i  <= we;

  muxpcwrite_reset : mux21 port map ('1', pcwrite, reset_l, pcwrite_reset);

  control : main_control port map (op_func => op, wb_ack => ack_int, overf => overflow, clk => clk, reset => reset_l,
                                   stb_o => stb, causewrite => causewrite, intcause => intcause, epcwrite => epcwrite,
                                   regwrite => regwrite, alusrca => alusrca, we =>  we, iord => iord, irwrite => irwrite,
                                   pcwrite => pcwrite,
                                   pcwritecond => pcwritecond, pcwbnq => pcwbnq, regdst => regdst, alusrcb => alusrcb,
                                   memtoreg => memtoreg,
                                   aluop => aluop, pcsource => pcsource);

  data_path : datapath port map (pcwritecond => pcwritecond, pcwbnq => pcwbnq, pcwrite => pcwrite_reset,
                                 iord => iord, irwrite => irwrite,
                                 alusrca => alusrca, regwrite => regwrite, clk => clk,
                                 causewrite => causewrite, intcause => intcause, epcwrite => epcwrite,
                                 aluop => aluop, alusrcb => alusrcb, pcsource => pcsource, regdst => regdst,
                                 memtoreg => memtoreg, reset => reset_l, memdata => memdata,
                                 cause => open, overflow => overflow, op => op, address => address,
                                 memwritedata => memwritedata, epcout => open);
  

end struct;


