-------------------------------------------------------------------------------
-- File name : regisetrfile.vhd
-- Title     : 32 x 32-bit Register-file 
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity registerfile is
  port (readreg1   : in  std_logic_vector(4 downto 0);
        readreg2   : in  std_logic_vector(4 downto 0);
         writereg  : in  std_logic_vector(4 downto 0);
         writedata : in  std_logic_vector(31 downto 0);
         regwrite  : in  std_logic;
         clk       : in  std_logic;
         readdata1 : out std_logic_vector(31 downto 0);
         readdata2 : out std_logic_vector(31 downto 0));
end registerfile;

architecture behavior of registerfile is

  component register32
    port (d   : in  std_logic_vector(31 downto 0);
          clk : in  std_logic;
          q   : out std_logic_vector(31 downto 0));
  end component;

  -- write signals to individual registers
  signal wr1, wr2, wr3, wr4, wr5, wr6, wr7, wr8, wr9 : std_logic;
  signal wr10, wr11, wr12, wr13, wr14, wr15, wr16, wr17, wr18, wr19 : std_logic;
  signal wr20, wr21, wr22, wr23, wr24, wr25, wr26, wr27, wr28, wr29 : std_logic;
  signal wr30, wr31 : std_logic;

  -- outputs (contents) of each register
  signal q1, q2, q3, q4, q5, q6, q7, q8, q9 : std_logic_vector(31 downto 0);
  signal q10, q11, q12, q13, q14, q15, q16, q17, q18, q19 : std_logic_vector(31 downto 0);
  signal q20, q21, q22, q23, q24, q25, q26, q27, q28, q29 : std_logic_vector(31 downto 0);
  signal q30, q31: std_logic_vector(31 downto 0);

begin
  reg1  : register32 port map(writedata, wr1, q1);
  reg2  : register32 port map(writedata, wr2, q2);
  reg3  : register32 port map(writedata, wr3, q3);
  reg4  : register32 port map(writedata, wr4, q4);
  reg5  : register32 port map(writedata, wr5, q5);
  reg6  : register32 port map(writedata, wr6, q6);
  reg7  : register32 port map(writedata, wr7, q7);
  reg8  : register32 port map(writedata, wr8, q8);
  reg9  : register32 port map(writedata, wr9, q9);
  reg10 : register32 port map(writedata, wr10, q10);
  reg11 : register32 port map(writedata, wr11, q11);
  reg12 : register32 port map(writedata, wr12, q12);
  reg13 : register32 port map(writedata, wr13, q13);
  reg14 : register32 port map(writedata, wr14, q14);
  reg15 : register32 port map(writedata, wr15, q15);
  reg16 : register32 port map(writedata, wr16, q16);
  reg17 : register32 port map(writedata, wr17, q17);
  reg18 : register32 port map(writedata, wr18, q18);
  reg19 : register32 port map(writedata, wr19, q19);
  reg20 : register32 port map(writedata, wr20, q20);
  reg21 : register32 port map(writedata, wr21, q21);
  reg22 : register32 port map(writedata, wr22, q22);
  reg23 : register32 port map(writedata, wr23, q23);
  reg24 : register32 port map(writedata, wr24, q24);
  reg25 : register32 port map(writedata, wr25, q25);
  reg26 : register32 port map(writedata, wr26, q26);
  reg27 : register32 port map(writedata, wr27, q27);
  reg28 : register32 port map(writedata, wr28, q28);
  reg29 : register32 port map(writedata, wr29, q29);
  reg30 : register32 port map(writedata, wr30, q30);
  reg31 : register32 port map(writedata, wr31, q31);
  
  -- write register signals for each register (register clock inputs)
  wr1  <= not writereg(4) and not writereg( 3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr2  <= not writereg(4) and not writereg( 3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr3  <= not writereg(4) and not writereg( 3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr4  <= not writereg(4) and not writereg( 3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr5  <= not writereg(4) and not writereg( 3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr6  <= not writereg(4) and not writereg( 3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr7  <= not writereg(4) and not writereg( 3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr8  <= not writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr9  <= not writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr10 <= not writereg(4) and writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr11 <= not writereg(4) and writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr12 <= not writereg(4) and writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr13 <= not writereg(4) and writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr14 <= not writereg(4) and writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr15 <= not writereg(4) and writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr16 <= writereg(4) and not writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr17 <= writereg(4) and not writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr18 <= writereg(4) and not writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr19 <= writereg(4) and not writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr20 <= writereg(4) and not writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr21 <= writereg(4) and not writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr22 <= writereg(4) and not writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr23 <= writereg(4) and not writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr24 <= writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr25 <= writereg(4) and writereg(3) and not writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr26 <= writereg(4) and writereg(3) and not writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr27 <= writereg(4) and writereg(3) and not writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;
  wr28 <= writereg(4) and writereg(3) and writereg(2) and not writereg(1) and not writereg(0) and regwrite and not clk;
  wr29 <= writereg(4) and writereg(3) and writereg(2) and not writereg(1) and writereg(0) and regwrite and not clk;
  wr30 <= writereg(4) and writereg(3) and writereg(2) and writereg(1) and not writereg(0) and regwrite and not clk;
  wr31 <= writereg(4) and writereg(3) and writereg(2) and writereg(1) and writereg(0) and regwrite and not clk;


  -- Behavior level implementation of mux32-1x32 (see single cycle for structural implementation)
  process (readreg1, readreg2)
  begin
    case readreg1 (4 downto 0) is
      when "00000" => readdata1 <= "00000000000000000000000000000000";
      when "00001" => readdata1 <= q1;
      when "00010" => readdata1 <= q2;
      when "00011" => readdata1 <= q3;
      when "00100" => readdata1 <= q4;
      when "00101" => readdata1 <= q5;
      when "00110" => readdata1 <= q6;
      when "00111" => readdata1 <= q7;
      when "01000" => readdata1 <= q8;
      when "01001" => readdata1 <= q9;
      when "01010" => readdata1 <= q10;
      when "01011" => readdata1 <= q11;
      when "01100" => readdata1 <= q12;
      when "01101" => readdata1 <= q13;
      when "01110" => readdata1 <= q14;
      when "01111" => readdata1 <= q15;
      when "10000" => readdata1 <= q16;
      when "10001" => readdata1 <= q17;
      when "10010" => readdata1 <= q18;
      when "10011" => readdata1 <= q19;
      when "10100" => readdata1 <= q20;
      when "10101" => readdata1 <= q21;
      when "10110" => readdata1 <= q22;
      when "10111" => readdata1 <= q23;
      when "11000" => readdata1 <= q24;
      when "11001" => readdata1 <= q25;
      when "11010" => readdata1 <= q26;
      when "11011" => readdata1 <= q27;
      when "11100" => readdata1 <= q28;
      when "11101" => readdata1 <= q29;
      when "11110" => readdata1 <= q30;
      when others  => readdata1 <= q31;
    end case;

    case readreg2(4 downto 0) is
      when "00000" => readdata2 <= "00000000000000000000000000000000";
      when "00001" => readdata2 <= q1;
      when "00010" => readdata2 <= q2;
      when "00011" => readdata2 <= q3;
      when "00100" => readdata2 <= q4;
      when "00101" => readdata2 <= q5;
      when "00110" => readdata2 <= q6;
      when "00111" => readdata2 <= q7;
      when "01000" => readdata2 <= q8;
      when "01001" => readdata2 <= q9;
      when "01010" => readdata2 <= q10;
      when "01011" => readdata2 <= q11;
      when "01100" => readdata2 <= q12;
      when "01101" => readdata2 <= q13;
      when "01110" => readdata2 <= q14;
      when "01111" => readdata2 <= q15;
      when "10000" => readdata2 <= q16;
      when "10001" => readdata2 <= q17;
      when "10010" => readdata2 <= q18;
      when "10011" => readdata2 <= q19;
      when "10100" => readdata2 <= q20;
      when "10101" => readdata2 <= q21;
      when "10110" => readdata2 <= q22;
      when "10111" => readdata2 <= q23;
      when "11000" => readdata2 <= q24;
      when "11001" => readdata2 <= q25;
      when "11010" => readdata2 <= q26;
      when "11011" => readdata2 <= q27;
      when "11100" => readdata2 <= q28;
      when "11101" => readdata2 <= q29;
      when "11110" => readdata2 <= q30;
      when others  => readdata2 <= q31;
    end case;

  end process;
end behavior;


