-- 32 Bit ALU (Based on RCA)
-- Author: John Moskal
--  Created: 11-25-04
-- Modified: 11-25-04
--
-- Modifications:
--
------------------------------------------------------------------------------------------
---------------------------------------------------------------------
---------------------------------------------------------------------

library work;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

entity ALU1 is
port
(
        A, B, L, Cin:   in std_logic;
        OP:             in std_logic_vector(2 downto 0);
        R, Cout:        out std_logic
);
end entity ALU1;

architecture stuff of ALU1 is

component MUX4
port
(       
        A, B, C, D:     in std_logic;
        SL:             in std_logic_vector(1 downto 0);
        R:              out std_logic
);
end component;

signal bnot:    std_logic;
signal wand:    std_logic;
signal wor:     std_logic;
signal wsum:    std_logic;
signal w1:      std_logic;
signal w2:      std_logic;

begin

        bnot <= (B xor (OP(2)));
        wand <= (A and bnot);
        wor <= (A or bnot);
        w1 <= ((not(wand)) and wor);
        w2 <= (w1 and Cin);
        wsum <= (not(w2)) and (Cin or w1);
        Cout <= (w2 or wand);

MUX:   MUX4 port map(wand, wor, wsum, L, OP(1 downto 0), R);

end stuff;

---------------------------------------------------------------------
---------------------------------------------------------------------
library work;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

entity ALU1L is
port
(
        A, B, L, Cin:   in std_logic;
        OP:             in std_logic_vector(2 downto 0);
        R, Cout, V, S:  out std_logic
);
end entity ALU1L;

architecture stuff of ALU1L is

component MUX4
port
(       
        A, B, C, D:     in std_logic;
        SL:             in std_logic_vector(1 downto 0);
        R:              out std_logic
);
end component;

signal bnot:    std_logic;
signal wand:    std_logic;
signal wor:     std_logic;
signal wsum:    std_logic;
signal wcout:   std_logic;
signal wv:      std_logic;
signal w1:      std_logic;
signal w2:      std_logic;

begin

        bnot <= (B xor (OP(2)));
        wand <= (A and bnot);
        wor <= (A or bnot);
        w1 <= ((not(wand)) and wor);
        w2 <= (w1 and Cin);

        wsum <= (not(w2)) and (Cin or w1);

        wcout <= (w2 or wand);
        Cout <= wcout;

        wv <= (Cin xor wcout);
        V <= wv;

        S <= (wv xor wsum);

MUX:   MUX4 port map(wand, wor, wsum, L, OP(1 downto 0), R);

end stuff;

---------------------------------------------------------------------
---------------------------------------------------------------------

library work;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

entity ALU32 is
port
(
        A, B:           in std_logic_vector(31 downto 0);
        SL:             in std_logic_vector(2 downto 0);
        R:              out std_logic_vector(31 downto 0);
        Z, V, C:        out std_logic
);
end entity ALU32;

architecture stuff of ALU32 is

component ALU1
port
(
        A, B, L, Cin:   in std_logic;
        OP:             in std_logic_vector(2 downto 0);
        R, Cout:        out std_logic
);      
end component;

component ALU1L
port
(
        A, B, L, Cin:   in std_logic;
        OP:             in std_logic_vector(2 downto 0);
        R, Cout, V, S:  out std_logic
);
end component;

signal wset:    std_logic;
signal wR:      std_logic_vector(31 downto 0);
signal wC:      std_logic_vector(31 downto 0);

begin
BIT00:  ALU1 port map(A(0),B(0),wset,SL(2),SL(2 downto 0),wR(0),wC(1));
BIT01:  ALU1 port map(A(1),B(1),'0',wC(1),SL(2 downto 0),wR(1),wC(2));
BIT02:  ALU1 port map(A(2),B(2),'0',wC(2),SL(2 downto 0),wR(2),wC(3));
BIT03:  ALU1 port map(A(3),B(3),'0',wC(3),SL(2 downto 0),wR(3),wC(4));
BIT04:  ALU1 port map(A(4),B(4),'0',wC(4),SL(2 downto 0),wR(4),wC(5));
BIT05:  ALU1 port map(A(5),B(5),'0',wC(5),SL(2 downto 0),wR(5),wC(6));
BIT06:  ALU1 port map(A(6),B(6),'0',wC(6),SL(2 downto 0),wR(6),wC(7));
BIT07:  ALU1 port map(A(7),B(7),'0',wC(7),SL(2 downto 0),wR(7),wC(8));
BIT08:  ALU1 port map(A(8),B(8),'0',wC(8),SL(2 downto 0),wR(8),wC(9));
BIT09:  ALU1 port map(A(9),B(9),'0',wC(9),SL(2 downto 0),wR(9),wC(10));
BIT10:  ALU1 port map(A(10),B(10),'0',wC(10),SL(2 downto 0),wR(10),wC(11));
BIT11:  ALU1 port map(A(11),B(11),'0',wC(11),SL(2 downto 0),wR(11),wC(12));
BIT12:  ALU1 port map(A(12),B(12),'0',wC(12),SL(2 downto 0),wR(12),wC(13));
BIT13:  ALU1 port map(A(13),B(13),'0',wC(13),SL(2 downto 0),wR(13),wC(14));
BIT14:  ALU1 port map(A(14),B(14),'0',wC(14),SL(2 downto 0),wR(14),wC(15));
BIT15:  ALU1 port map(A(15),B(15),'0',wC(15),SL(2 downto 0),wR(15),wC(16));
BIT16:  ALU1 port map(A(16),B(16),'0',wC(16),SL(2 downto 0),wR(16),wC(17));
BIT17:  ALU1 port map(A(17),B(17),'0',wC(17),SL(2 downto 0),wR(17),wC(18));
BIT18:  ALU1 port map(A(18),B(18),'0',wC(18),SL(2 downto 0),wR(18),wC(19));
BIT19:  ALU1 port map(A(19),B(19),'0',wC(19),SL(2 downto 0),wR(19),wC(20));
BIT20:  ALU1 port map(A(20),B(20),'0',wC(20),SL(2 downto 0),wR(20),wC(21));
BIT21:  ALU1 port map(A(21),B(21),'0',wC(21),SL(2 downto 0),wR(21),wC(22));
BIT22:  ALU1 port map(A(22),B(22),'0',wC(22),SL(2 downto 0),wR(22),wC(23));
BIT23:  ALU1 port map(A(23),B(23),'0',wC(23),SL(2 downto 0),wR(23),wC(24));
BIT24:  ALU1 port map(A(24),B(24),'0',wC(24),SL(2 downto 0),wR(24),wC(25));
BIT25:  ALU1 port map(A(25),B(25),'0',wC(25),SL(2 downto 0),wR(25),wC(26));
BIT26:  ALU1 port map(A(26),B(26),'0',wC(26),SL(2 downto 0),wR(26),wC(27));
BIT27:  ALU1 port map(A(27),B(27),'0',wC(27),SL(2 downto 0),wR(27),wC(28));
BIT28:  ALU1 port map(A(28),B(28),'0',wC(28),SL(2 downto 0),wR(28),wC(29));
BIT29:  ALU1 port map(A(29),B(29),'0',wC(29),SL(2 downto 0),wR(29),wC(30));
BIT30:  ALU1 port map(A(30),B(30),'0',wC(30),SL(2 downto 0),wR(30),wC(31));
BIT31:  ALU1L port map(A(31),B(31),'0',wC(31),SL(2 downto 0),wR(31),C,V,wset);

R <= wR;

process (wR)
begin
if (wR = "00000000000000000000000000000000") then 
        Z <= '1';
else
        Z <= '0';
end if;
end process;

end stuff;

---------------------------------------------------------------------
---------------------------------------------------------------------


